/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


import org.intellij.plugins.xsltDebugger.rt.engine.BreakpointManager;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.rt.engine.local.LocalDebugger;

import com.icl.saxon.TransformerFactoryImpl;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;

public class Main {
    public static void main(String[] args) throws TransformerException, IOException, InterruptedException {
        final TransformerFactoryImpl factory = new TransformerFactoryImpl();

        final File f = new File("test.xsl");
        final File x = new File("test.xml");
        final Transformer transformer = factory.newTransformer(new StreamSource(f));

        final StreamSource source = new StreamSource(x);
        final StreamResult result = new StreamResult();

        final Debugger debugger = new LocalDebugger(transformer, source, result);

        final BreakpointManager manager = debugger.getBreakpointManager();
//        manager.setXsltBreakpoint(f, 16);
        manager.setBreakpoint(f, 13);

        debugger.start();

        final BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(System.in));
        do {
            if (debugger.waitForDebuggee()) {
                System.out.println("Stopped at #" + debugger.getCurrentFrame().getLineNumber());
                System.out.print("> ");

                do {
                    final String s = bufferedReader.readLine();
                    if ("stop".equalsIgnoreCase(s)) {
                        debugger.stop(false);
                        break;
                    } else if ("step".equalsIgnoreCase(s)) {
                        debugger.step();
                        break;
                    } else if ("stepinto".equalsIgnoreCase(s)) {
                        debugger.stepInto();
                        break;
                    } else if ("go".equalsIgnoreCase(s)) {
                        debugger.resume();
                        break;
                    } else if (s != null && s.toLowerCase().startsWith("eval ")) {
                        try {
                            final String expr = s.substring("eval ".length());
                            System.out.println(expr + " = " + debugger.eval(expr));
                        } catch (Debugger.EvaluationException e) {
                            System.err.println("Error in expression: " + e.getMessage());
                        }
                    } else if ("variables".equals(s)) {
                        List<Debugger.Variable> vars = debugger.getCurrentFrame().getVariables();
                        System.out.println("vars = " + vars);
                    } else if (s == null) {
                        debugger.stop(true);
                        break;
                    } else if (s.length() == 0) {
                        System.out.println();
                    } else {
                        System.out.println("Unknown command: " + s);
                    }
                } while (true);
            }

        } while (!debugger.isStopped());

        System.out.println("Done.");
        
        System.exit(0);
    }
}
