/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.intellij.plugins.xsltDebugger;

import org.intellij.plugins.xsltDebugger.ui.XsltBreakpointSettings;

import com.intellij.debugger.ui.breakpoints.Breakpoint;
import com.intellij.debugger.ui.breakpoints.BreakpointFactory;
import com.intellij.debugger.ui.breakpoints.BreakpointPanel;
import com.intellij.debugger.ui.breakpoints.BreakpointPropertiesPanel;
import com.intellij.debugger.ui.breakpoints.LineBreakpoint;
import com.intellij.debugger.ui.breakpoints.actions.BreakpointPanelAction;
import com.intellij.debugger.ui.breakpoints.actions.GotoSourceAction;
import com.intellij.debugger.ui.breakpoints.actions.RemoveAction;
import com.intellij.debugger.ui.breakpoints.actions.ViewSourceAction;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.DialogWrapper;
import com.intellij.openapi.util.Key;

import org.intellij.lang.xpath.xslt.XsltSupport;
import org.jdom.Element;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;

/**
 * Created by IntelliJ IDEA.
 * User: sweinreuter
 * Date: 26.05.2007
 */
public class XsltBreakpointFactory extends BreakpointFactory {
    private static final String DISPLAY_NAME = "XSLT Breakpoints";

    public Breakpoint createBreakpoint(Project project, final Element element) {
      return new XsltBreakpoint(project);
    }

    public Icon getIcon() {
      return getIconStatic();
    }

    public static Icon getIconStatic() {
        return XsltSupport.createXsltIcon(LineBreakpoint.ICON);
    }

    public Icon getDisabledIcon() {
      return getDisabledIconStatic();
    }

    public static Icon getDisabledIconStatic() {
        return XsltSupport.createXsltIcon(LineBreakpoint.DISABLED_ICON);
    }

    public BreakpointPanel createBreakpointPanel(Project project, final DialogWrapper parentDialog) {
        return new BreakpointPanel(project,
                new MyPropertiesPanel(project),
                new BreakpointPanelAction[]{
                        new GotoSourceAction(project) {
                            public void actionPerformed(ActionEvent e) {
                                super.actionPerformed(e);
                                parentDialog.close(DialogWrapper.OK_EXIT_CODE);
                            }
                        },
                        new ViewSourceAction(project),
                        new RemoveAction(project),
                },
                getBreakpointCategory(), DISPLAY_NAME, null);
    }

    public Key<LineBreakpoint> getBreakpointCategory() {
      return XsltBreakpoint.CATEGORY;
    }

    private static class MyPropertiesPanel extends BreakpointPropertiesPanel {
        private final XsltBreakpointSettings mySettings;

        public MyPropertiesPanel(Project project) {
            super(project, XsltBreakpoint.CATEGORY);
            mySettings = new XsltBreakpointSettings(project);

            final JPanel panel = getPanel();
            panel.removeAll(); // we don't need anything from the normal, java-specific panel
            panel.setLayout(new BorderLayout());
            panel.add(mySettings.getRoot(), BorderLayout.CENTER);
        }

        public void initFrom(Breakpoint breakpoint) {
            super.initFrom(breakpoint);
            mySettings.initFrom((XsltBreakpoint)breakpoint);
        }

        public void saveTo(Breakpoint breakpoint, Runnable afterUpdate) {
            super.saveTo(breakpoint, afterUpdate);
            mySettings.saveTo((XsltBreakpoint)breakpoint);
        }
    }
}
