/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions;

import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;

import com.intellij.psi.xml.XmlElement;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import org.intellij.lang.xpath.context.ContextProvider;
import org.intellij.lang.xpath.context.ContextType;
import org.intellij.lang.xpath.context.AbstractVariableContext;
import org.intellij.lang.xpath.context.NamespaceContext;
import org.intellij.lang.xpath.context.VariableContext;
import org.intellij.lang.xpath.psi.XPathElement;
import org.intellij.lang.xpath.xslt.context.XsltContextProvider;

import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: sweinreuter
 * Date: 06.06.2007
 */
public class EvalContextProvider extends ContextProvider {
    private final List<Debugger.Variable> myVariables;

    public EvalContextProvider(List<Debugger.Variable> model) {
        myVariables = model;
    }

    @NotNull
    public ContextType getContextType() {
        return XsltContextProvider.TYPE;
    }

    @Nullable
    public XmlElement getContextElement() {
        return null;
    }

    @Override
    protected boolean isValid() {
        return true;
    }

    @Nullable
    public NamespaceContext getNamespaceContext() {
        return null;
    }

    public VariableContext getVariableContext() {
        return new AbstractVariableContext<String>() {
            @NotNull
            public String[] getVariablesInScope(XPathElement element) {
                final int size = myVariables.size();
                final ArrayList<String> vars = new ArrayList<String>(size);
                for (int i = 0; i < size; i++) {
                    vars.add(myVariables.get(i).getName());
                }
                return vars.toArray(new String[vars.size()]);
            }
        };
    }

    @Nullable
    public Set<QName> getAttributes(boolean forValidation) {
        return null; // TODO
    }

    @Nullable
    public Set<QName> getElements(boolean forValidation) {
        return null; // TODO
    }
}
