/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions;

import org.intellij.plugins.xsltDebugger.XsltDebuggerSession;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.ui.DebuggerTabComponent;
import org.intellij.plugins.xsltDebugger.ui.EvaluationDialog;

import com.intellij.debugger.actions.DebuggerActions;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.project.Project;

/**
 * Created by IntelliJ IDEA.
* User: sweinreuter
* Date: 28.11.2007
*/
public class EvaluateExpressionAction extends SuspendedDebuggerAction {
    public EvaluateExpressionAction() {
        copyIcon(DebuggerActions.EVALUATE_EXPRESSION);
    }

    public void actionPerformedImpl(AnActionEvent e, XsltDebuggerSession session) {
        final Project project = DataKeys.PROJECT.getData(e.getDataContext());
        Editor editor = DataKeys.EDITOR.getData(e.getDataContext());
        if (editor == null) {
            editor = FileEditorManager.getInstance(project).getSelectedTextEditor();
        }

        final DebuggerTabComponent component = session.getTabComponent();
        final String selection = editor != null ? editor.getSelectionModel().getSelectedText() : null;
        final EvalContextProvider context = new EvalContextProvider(component.getVariablesModel().getVariables());

        final Debugger.StyleFrame frame = component.getSelectedFrame();
        final EvaluationDialog dialog = new EvaluationDialog(project, selection, context, frame);
        dialog.show();
    }
}
