/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.Presentation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/*
* Created by IntelliJ IDEA.
* User: sweinreuter
* Date: 29.11.2007
*/
abstract class HookedDebuggerAction extends AnAction {
    private final AnAction myOrigAction;

    public HookedDebuggerAction(AnAction origAction) {
        myOrigAction = origAction;
        copyFrom(origAction);
        setEnabledInModalContext(origAction.isEnabledInModalContext());
    }

    public final AnAction getOrigAction() {
        return myOrigAction;
    }

    public final void update(AnActionEvent e) {
        myOrigAction.update(e);
        final Presentation presentation = e.getPresentation();
        if (!presentation.isEnabled()) {
            presentation.setEnabled(isEnabled(e));
        }
    }

    protected abstract boolean isEnabled(AnActionEvent e);

    public final void actionPerformed(AnActionEvent e) {
        if (!actionPerformedImpl(e)) {
            try {
                myOrigAction.actionPerformed(e);
            } catch (RuntimeException e1) {
                // That one is a bit nasty, but I don't like to be blamed for IDEA core errors ;)
                //noinspection ThrowableInstanceNeverThrown
                final StackTraceElement here = new Throwable().getStackTrace()[0];
                final String ourClassName = here.getClassName();
                final String ourMethodName = here.getMethodName();

                final List<StackTraceElement> list = Arrays.asList(e1.getStackTrace());
                final List<StackTraceElement> trace = new ArrayList<StackTraceElement>(list);
                for (Iterator<StackTraceElement> it = trace.iterator(); it.hasNext();) {
                    final StackTraceElement element = it.next();
                    if (ourClassName.equals(element.getClassName()) &&
                            ourMethodName.equals(element.getMethodName())) {

                        it.remove();
                        e1.setStackTrace(trace.toArray(new StackTraceElement[trace.size()]));
                        throw e1;
                    }
                }
                throw e1;
            }
        }
    }

    protected abstract boolean actionPerformedImpl(AnActionEvent e);

    public boolean displayTextInToolbar() {
        return myOrigAction.displayTextInToolbar();
    }

    public void setDefaultIcon(boolean b) {
        myOrigAction.setDefaultIcon(b);
    }

    public boolean isDefaultIcon() {
        return myOrigAction.isDefaultIcon();
    }

    public void setInjectedContext(boolean worksInInjected) {
        myOrigAction.setInjectedContext(worksInInjected);
    }

    public boolean isInInjectedContext() {
        return myOrigAction.isInInjectedContext();
    }
}