/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions;

import org.intellij.plugins.xsltDebugger.XsltDebuggerSession;
import org.intellij.plugins.xsltDebugger.XsltDebuggerSessionTracker;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.project.Project;

/**
 * Created by IntelliJ IDEA.
* User: sweinreuter
* Date: 28.11.2007
*/
public abstract class XsltDebuggerAction extends AnAction {
    public static final String CONTROL_GROUP = "XSLT-Debugger.Control";
    public static final String STEP_INTO = CONTROL_GROUP + ".StepInto";
    public static final String STEP_OVER = CONTROL_GROUP + ".StepOver";
    public static final String RUN_TO_CURSOR = CONTROL_GROUP + ".RunToCursor";
    public static final String RESUME = CONTROL_GROUP + ".Resume";
    public static final String PAUSE = CONTROL_GROUP + ".Pause";

    public static final String WATCHES_GROUP = "XSLT-Debugger.Watches";
    public static final String EDIT_WATCH = WATCHES_GROUP + ".Edit";

    public static final String EVALUATE_EXPRESSION = "XSLT-Debugger.EvaluateExpression";

    @Override
    public final void update(AnActionEvent e) {
        e.getPresentation().setEnabled(isEnabled(e));
    }

    protected boolean isEnabled(AnActionEvent e) {
        final Project project = e.getData(DataKeys.PROJECT);
        if (project == null) {
            return false;
        }
        final XsltDebuggerSession session = XsltDebuggerSessionTracker.getInstance(project).getActiveSession();
        if ((session != null && session.isVmActive())) {
            synchronized (session.getTabComponent().myLock) {
                return isEnabled(e, session);
            }
        }
        return false;
    }

    protected boolean isEnabled(AnActionEvent e, XsltDebuggerSession session) {
        return acceptsState(session.getCurrentState());
    }

    protected abstract boolean acceptsState(Debugger.State state);

    public final void actionPerformed(AnActionEvent e) {
        final XsltDebuggerSession session = XsltDebuggerSessionTracker.getInstance(e.getData(DataKeys.PROJECT)).getActiveSession();
        if ((session != null && session.isVmActive())) {
            synchronized (session.getTabComponent().myLock) {
                actionPerformedImpl(e, session);
            }
        }
    }

    protected abstract void actionPerformedImpl(AnActionEvent e, XsltDebuggerSession session);

    protected final void copyIcon(String s) {
        getTemplatePresentation().setIcon(ActionManager.getInstance().getAction(s).getTemplatePresentation().getIcon());
    }
}
