/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions.control;

import org.intellij.plugins.xsltDebugger.XsltDebuggerSession;
import org.intellij.plugins.xsltDebugger.actions.SuspendedDebuggerAction;

import com.intellij.debugger.actions.DebuggerActions;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiFile;

import org.intellij.lang.xpath.xslt.XsltSupport;

public class RunToCursorAction extends SuspendedDebuggerAction {
    public RunToCursorAction() {
        copyIcon(DebuggerActions.RUN_TO_CURSOR);
    }

    public void actionPerformedImpl(AnActionEvent e, XsltDebuggerSession session) {
        final Project project = e.getData(DataKeys.PROJECT);
        assert project != null;

        final Editor editor = FileEditorManager.getInstance(project).getSelectedTextEditor();
        assert editor != null;

        final Document document = editor.getDocument();
        final PsiFile file = PsiDocumentManager.getInstance(project).getPsiFile(document);
        final int offset = editor.getCaretModel().getOffset();
        assert file != null;
        session.runTo(file, document, offset);
    }

    protected boolean isEnabled(AnActionEvent e, XsltDebuggerSession session) {
        if (!super.isEnabled(e, session)) return false;

        final Project project = e.getData(DataKeys.PROJECT);
        if (project == null) {
            return false;
        }
        final Editor editor = FileEditorManager.getInstance(project).getSelectedTextEditor();
        if (editor != null) {
            final PsiFile file = PsiDocumentManager.getInstance(project).getPsiFile(editor.getDocument());
            if (file != null) {
                final int offset = editor.getCaretModel().getOffset();
                return XsltSupport.isXsltFile(file) && file.getVirtualFile() != null &&
                        session.canRunTo(file, editor.getDocument(), offset);
            }
        }
        return false;
    }
}