/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.actions.watches;

import org.intellij.plugins.xsltDebugger.XsltDebuggerSession;
import org.intellij.plugins.xsltDebugger.actions.EvalContextProvider;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.ui.DebuggerTabComponent;
import org.intellij.plugins.xsltDebugger.ui.EvaluationDialog;
import org.intellij.plugins.xsltDebugger.ui.VariablesModel;

import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.project.Project;

import javax.swing.tree.TreeNode;
import java.util.Collections;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
* User: sweinreuter
* Date: 28.11.2007
*/
public class EditWatchAction extends AbstractWatchAction {

    public void actionPerformedImpl(AnActionEvent e, XsltDebuggerSession session) {
        final Project project = e.getData(DataKeys.PROJECT);
        if (project == null) return;

        final TreeNode o = getVarNode(e);
        if (o != null) {
            final DebuggerTabComponent component = session.getTabComponent();
            final Debugger.StyleFrame frame = component.getSelectedFrame();
            final List<Debugger.Variable> vars = frame != null ? frame.getVariables() : Collections.<Debugger.Variable>emptyList();
            final EvalContextProvider provider = new EvalContextProvider(vars);

            final Debugger.Variable variable = ((VariablesModel.VariableNode)o).getUserObject();
            final EvaluationDialog dialog = new EvaluationDialog(project, variable.getName(), provider, "Edit Watch");
            dialog.show();

            if (dialog.isOK()) {
                component.getWatchesModel().editWatch(variable, dialog.getExpression(), frame);
            }
        }
    }
}
