/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.ui;

import org.intellij.plugins.xsltDebugger.VMPausedException;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger.Frame;

import com.intellij.openapi.vfs.VfsUtil;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.ui.ColoredListCellRenderer;
import com.intellij.ui.SimpleTextAttributes;

import javax.swing.*;
import java.net.URI;

/**
 * Created by IntelliJ IDEA.
* User: sweinreuter
* Date: 31.05.2007
*/
class FrameCellRenderer extends ColoredListCellRenderer {
    protected void customizeCellRenderer(JList list, Object value, int index, boolean selected, boolean hasFocus) {
        try {
            customizeCellRendererImpl(value);
        } catch (VMPausedException e) {
            append("Information unavailable", SimpleTextAttributes.ERROR_ATTRIBUTES);
        }
    }

    protected void customizeCellRendererImpl(Object value) {
        final Frame frame = (Frame)value;
        if (value instanceof Debugger.StyleFrame) {
            append(((Debugger.StyleFrame)frame).getInstruction(), SimpleTextAttributes.REGULAR_BOLD_ATTRIBUTES);
        } else if (frame instanceof Debugger.SourceFrame) {
            append(((Debugger.SourceFrame)frame).getXPath(), SimpleTextAttributes.REGULAR_BOLD_ATTRIBUTES);
        }
        append(" ", SimpleTextAttributes.REGULAR_ATTRIBUTES);

        try {
            final VirtualFile file = VfsUtil.findFileByURL(new URI(frame.getURI()).toURL());
            if (file != null) {
                append(file.getName(), SimpleTextAttributes.REGULAR_ATTRIBUTES);
                if (frame.getLineNumber() > 0) {
                    append(":" + frame.getLineNumber(), SimpleTextAttributes.REGULAR_ATTRIBUTES);
                }

                setToolTipText(file.getPresentableUrl());
            } else {
                append(frame.getURI() + ":" + frame.getLineNumber(), SimpleTextAttributes.REGULAR_ATTRIBUTES);
            }
        } catch (Exception e) {
            append(frame.getURI() + ":" + frame.getLineNumber(), SimpleTextAttributes.REGULAR_ATTRIBUTES);
        }
    }
}
