/*
 * Copyright 2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.ui;

import org.intellij.plugins.xsltDebugger.VMPausedException;
import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.rt.engine.Value;

import com.intellij.ui.ColoredTreeCellRenderer;
import com.intellij.ui.SimpleTextAttributes;
import com.intellij.util.Icons;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: sweinreuter
 * Date: 31.05.2007
 */
class VariableRenderer extends ColoredTreeCellRenderer {
    private static final SimpleTextAttributes NAME_ATTRIBUTES = new SimpleTextAttributes(SimpleTextAttributes.STYLE_PLAIN, new Color(128, 0, 0));

    public void customizeCellRenderer(JTree tree, Object node, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        try {
            customizeCellRendererImpl(node);
        } catch (VMPausedException e) {
            append("Information unvailable", SimpleTextAttributes.ERROR_ATTRIBUTES);
        }
    }

    public void customizeCellRendererImpl(Object node) {
        final Object value = ((DefaultMutableTreeNode)node).getUserObject();

        if (value == null) {
            return;
        }
        if (value instanceof Debugger.Variable) {
            final Debugger.Variable variable = (Debugger.Variable)value;
            final Debugger.Variable.Kind kind = variable.getKind();
            if (variable.isGlobal()) {
                if (kind == Debugger.Variable.Kind.VARIABLE) {
                    setIcon(Icons.FIELD_ICON);
                } else {
                    setIcon(Icons.PROPERTY_ICON);
                }
            } else if (kind == Debugger.Variable.Kind.VARIABLE) {
                setIcon(Icons.VARIABLE_ICON);
            } else if (kind == Debugger.Variable.Kind.PARAMETER) {
                setIcon(Icons.PARAMETER_ICON);
            }

            append(variable.getName(), NAME_ATTRIBUTES);
            append(" = ", SimpleTextAttributes.REGULAR_ATTRIBUTES);

            final Value v = variable.getValue();
            if (v instanceof WatchesModel.BadValue) {
                append(String.valueOf(v.getValue()), SimpleTextAttributes.ERROR_ATTRIBUTES);
            } else if (node instanceof VariablesModel.NodeSetNode) {
                final VariablesModel.NodeSetNode n = (VariablesModel.NodeSetNode)node;
                final String stringValue = n.getStringValue();
                append("'", SimpleTextAttributes.REGULAR_ATTRIBUTES);
                append(clipValue(stringValue), SimpleTextAttributes.REGULAR_ATTRIBUTES);
                append("'", SimpleTextAttributes.REGULAR_ATTRIBUTES);
            } else {
                if (v.getType() == Value.XPathType.STRING) {
                    append("'", SimpleTextAttributes.REGULAR_ATTRIBUTES);
                    append(String.valueOf(v.getValue()), SimpleTextAttributes.REGULAR_ATTRIBUTES);
                    append("'", SimpleTextAttributes.REGULAR_ATTRIBUTES);
                } else {
                    append(String.valueOf(v.getValue()), SimpleTextAttributes.REGULAR_ATTRIBUTES);
                }
            }
            append(" (" + v.getType().getName() + ")", SimpleTextAttributes.GRAYED_ATTRIBUTES);
        } else if (value instanceof Value.Node) {
            final Value.Node n = (Value.Node)value;
            append(n.myXPath, SimpleTextAttributes.REGULAR_ATTRIBUTES);
            append(" ('" + clipValue(n.myStringValue) + "')", SimpleTextAttributes.GRAYED_ATTRIBUTES);
            setToolTipText(n.myURI + ":" + n.myLineNumber);
        }
    }

    static String clipValue(String stringValue) {
        return stringValue.length() < 80 ? stringValue : stringValue.substring(0, 80) + "...";
    }
}
