/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.intellij.plugins.xsltDebugger.ui;

import org.intellij.plugins.xsltDebugger.ui.actions.CopyValueAction;
import org.intellij.plugins.xsltDebugger.ui.actions.NavigateAction;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.DataKey;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.actionSystem.DataSink;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.TypeSafeDataProvider;
import com.intellij.pom.Navigatable;
import com.intellij.ui.PopupHandler;
import com.intellij.util.ui.Tree;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

/**
 * Created by IntelliJ IDEA.
 * User: sweinreuter
 * Date: 17.06.2007
 */
class VariablesTree extends Tree implements TypeSafeDataProvider {

    public VariablesTree() {
        init();
    }
    
    public VariablesTree(VariablesModel model) {
        super(model);

        init();
    }

    private void init() {
        setRootVisible(false);
        setCellRenderer(new VariableRenderer());
        setShowsRootHandles(true);

        final DefaultActionGroup variableContextActions = new DefaultActionGroup("VariableContext", true);
        variableContextActions.add(NavigateAction.getInstance());
        variableContextActions.add(new CopyValueAction(this));
        PopupHandler.installFollowingSelectionTreePopup(this, variableContextActions, "XSLT.Debugger.Variables", ActionManager.getInstance());
    }

    public void calcData(DataKey key, DataSink sink) {
        if (key.equals(DataKeys.NAVIGATABLE)) {
            final TreePath selection = getSelectionPath();
            if (selection != null) {
                final Object o = selection.getLastPathComponent();
                if (o instanceof Navigatable) {
                    sink.put(DataKeys.NAVIGATABLE, (Navigatable)o);
                }
            }
        } else if (key.equals(CopyValueAction.SELECTED_NODE)) {
            final TreePath selection = getSelectionPath();
            if (selection != null) {
                final Object o = selection.getLastPathComponent();
                sink.put(CopyValueAction.SELECTED_NODE, (DefaultMutableTreeNode)o);
            }
        }
    }
}
