/*
 * Copyright 2002-2007 Sascha Weinreuter
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.intellij.plugins.xsltDebugger.ui.actions;

import org.intellij.plugins.xsltDebugger.rt.engine.Debugger;
import org.intellij.plugins.xsltDebugger.rt.engine.Value;
import org.intellij.plugins.xsltDebugger.ui.VariablesModel;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKey;
import com.intellij.openapi.ide.CopyPasteManager;

import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.datatransfer.StringSelection;

@SuppressWarnings({ "ComponentNotRegistered" })
public class CopyValueAction extends AnAction {
    public static final DataKey<DefaultMutableTreeNode> SELECTED_NODE = DataKey.create("SELECTED_NODE");

    public CopyValueAction(JComponent component) {
        final AnAction action = ActionManager.getInstance().getAction("$Copy");
        if (action != null) {
            copyFrom(action);
            registerCustomShortcutSet(getShortcutSet(), component);
        }
    }

    @Override
    public void update(AnActionEvent e) {
        e.getPresentation().setEnabled(isEnabled(e));
    }

    public void actionPerformed(AnActionEvent e) {
        final DefaultMutableTreeNode node = e.getData(SELECTED_NODE);
        if (node instanceof VariablesModel.NodeInfoNode) {
            final VariablesModel.NodeInfoNode infoNode = (VariablesModel.NodeInfoNode)node;
            final Value.Node info = infoNode.getNodeInfo();
            final String value = info.myStringValue;
            if (value != null) {
                setClipboardData(value);
            }
        } else if (node instanceof VariablesModel.NodeSetNode) {
            final VariablesModel.NodeSetNode nodeSetNode = (VariablesModel.NodeSetNode)node;

            final String s = nodeSetNode.getStringValue();
            if (s != null) {
                setClipboardData(s);
            }
        } else if (node != null && node.getUserObject() instanceof Debugger.Variable) {
            final Debugger.Variable variable = (Debugger.Variable)node.getUserObject();

            final String data = String.valueOf(variable.getValue().getValue());
            setClipboardData(data);
        }
    }

    private static void setClipboardData(String value) {
        CopyPasteManager.getInstance().setContents(new StringSelection(value));
    }

    protected boolean isEnabled(AnActionEvent e) {
        final DefaultMutableTreeNode node = e.getData(SELECTED_NODE);
        if (node instanceof VariablesModel.NodeInfoNode) {
            final VariablesModel.NodeInfoNode infoNode = (VariablesModel.NodeInfoNode)node;
            final Value.Node info = infoNode.getNodeInfo();
            return info.myStringValue != null;
        } else if (node instanceof VariablesModel.NodeSetNode) {
            return ((VariablesModel.NodeSetNode)node).getStringValue() != null;
        } else if (node != null) {
            return node.getUserObject() instanceof Debugger.Variable;
        }
        return false;
    }
}
